@@ -53,41 +53,87 @@ sub set_url_encoding(Str $encoding) is export {
 # utility functions
 
 sub header (
-    +$status = '200 OK', 
-    +$content_type = 'text/html', 
-    +$charset,
-    +$cookies,
-    +$location
+    Str ?$content_type = 'text/html',
+    Str ?$status = '200 OK',
+    Str ?$charset,
+    Str +$cookies,
+    Str +$target,
+    +$expires,
+    Bool +$nph,
+    *%extra
 ) returns Str is export {
     # construct our header
     my $header;
-    $header ~= "Status: " ~ $status ~ "\n";
+    $header ~= "Status: " ~ $status;
     # TODO:
     # Need to add support for -
+    #    NPH
     #    Expires:
     #    Pragma: (caching)
+    
+    $header ~= "\nContent-Type: " ~ $content_type;
+    $header ~= "; charset=$charset" if $charset.defined;
+    
+    for %extra.kv -> $key, $value {
+        # XXX use $key is rw;
+        my $temp_key = ucfirst(lc($key));
+        
+        $temp_key ~~ s:P5:g/[-_](\w)/-$0.uc()/;
+        
+        given $key {
+            when "Target" { $header ~= "\nWindow-Target: " ~ $value; }
+            default { $header ~= "\n" ~ $temp_key ~ ": " ~ $value; }
+        }
+    }
+    
     if ($cookies) {
         my @cookies = ($cookies !~ Array) ?? ($cookies) :: @{$cookies};
         
         for @cookies -> $cookie {
             #$cookie = ($cookie ~~ CGI::Cookie) ?? $cookie.as_string :: $cookie;
             
-            $header ~= "Set-Cookie: " ~ $cookie ~ "\n"
+            $header ~= "\nSet-Cookie: " ~ $cookie
                 unless $cookie eq "";
         }
     }
-    if ($location.defined) {
-        $header ~= "Location: " ~ $location;
-    }
-    else {
-        $header ~= "Content-type: " ~ $content_type;
-        $header ~= "; charset=$charset" if $charset.defined;        
-    }
+    
     return "$header\n\n";
 }
 
-sub redirect (Str $location) returns Str is export { 
-    header(status => '302 Moved', location => $location) 
+sub redirect (
+    Str $location,
+    Str ?$target,
+    Str ?$status = "302 Found",
+    Str +$cookie,
+    Bool +$nph,
+    *%extra
+) returns Str is export {
+    my %out;
+    
+    # XXX provide default for $location
+    #$location //= $self.location;
+    
+    # XXX just clone %extra
+    #%out = %extra.clone();
+    
+    %out<Location> = $location;
+    
+    for %extra.kv -> $header, $value {
+        %out{$header} = $value;
+    }
+    
+    if ($target.defined) { %out<Target> = $target; }
+    
+    for %out.keys -> $key {
+        %out{$key} = unescapeHTML(%out{$key})
+            unless $key eq "Cookie";
+    }
+    
+    if ($cookie.defined) {
+        return header('', $status, cookies => $cookie, nph => $nph, extra => %out);
+    } else {
+        return header('', $status, nph => $nph, extra => %out);
+    }
 }
 
 sub url_decode (Str $to_decode) returns Str is export {
@@ -193,6 +239,67 @@ sub load_params {
     }    
 }
 
+sub escapeHTML (Str $string, Bool +$newlines) returns Str is export {
+    # XXX check for $self.escape == 0
+    #unless ($self.escape != 0) { return $toencode; }
+    
+    $string ~~ s:P5:g/&/&amp;/;
+    $string ~~ s:P5:g/</&lt;/;
+    $string ~~ s:P5:g/>/&gt;/;
+    
+    # XXX check for HTML 3.2
+    #if ($self.DTD_PUBLIC_IDENTIFIER ~~ rx:P5/[^X]HTML 3\.2/i) {
+        # $quot; was accidentally omitted from the HTML 3.2 DTD -- see
+        # <http://validator.w3.org/docs/errors.html#bad-entity> /
+        # <http://lists.w3.org/Archives/Public/www-html/1997Mar/0003.html>.
+        
+        #$string ~~ s:P5:g/"/&#34;/;
+    #} else {
+        $string ~~ s:P5:g/"/&quot;/;
+    #}
+    
+    my $latin;
+    
+    # XXX check $self.charset
+    #$latin = ?(uc $self.charset eq "ISO-8859-1"|"WINDOWS-1252");
+    $latin = 1;
+    
+    if ($latin) {
+        $string ~~ s:P5:g/'/&#39;/;
+        $string ~~ s:P5:g/\x8b/&#8249;/;
+        $string ~~ s:P5:g/\x9b/&#8250;/;
+        
+        if ($newlines) {
+            $string ~~ s:P5:g/\012/&#10;/;
+            $string ~~ s:P5:g/\015/&#13;/;
+        }
+    }
+    
+    return $string;
+}
+
+sub unescapeHTML (Str $string) returns Str is export {
+    # XXX check $self.charset
+    #my $latin = ?(uc $self.charset eq "ISO-8859-1"|"WINDOWS-1252");
+    my $latin = 1;
+    
+    $string ~~ s:P5:g/&([^;]*);/{
+        given (lc $1) {
+            when "amp"  { "&" }
+            when "quot" { '"' }
+            when "gt"   { ">" }
+            when "lt"   { "<" }
+            
+            if ($latin) {
+                when /^#(\d+)$/     { chr($1) }
+                when /^#x(\d+)$/    { chr(hex($1)) }
+            }
+        }
+    }/;
+    
+    return $string;
+}
+
 # information functions (again)
 
 multi sub param returns Array is export { unless($IS_PARAMS_LOADED) {load_params}; %PARAMS.keys; }
@@ -303,7 +410,7 @@ Curtis "Ovid" Poe
  
 Andras Barthazi, E<lt>andras@barthazi.huE<gt>
 
-"Aankhen``"
+"Aankhen"
 
 =head1 COPYRIGHT
 
