@@ -41,10 +41,10 @@ sub splitpath (Str $path, Bool ?$nofile) returns Array is export {
 
 sub catdir (*@_path) returns Str is export {
     return '' unless +@_path;
-    # take a copy of our args here, maybe 
-    # replace this with 'is copy' parameter 
+    # take a copy of our args here, maybe
+    # replace this with 'is copy' parameter
     # trait at some point
-    my @path = @_path; 
+    my @path = @_path;
     my @new_path;
     my $i = 0;
     loop ($i = 0; $i < @path; $i++) {
@@ -60,9 +60,9 @@ sub catdir (*@_path) returns Str is export {
 }
 
 sub catfile (*@_path) returns Str is export {
-    # take a copy of our args here, maybe 
-    # replace this with 'is copy' parameter 
-    # trait at some point    
+    # take a copy of our args here, maybe
+    # replace this with 'is copy' parameter
+    # trait at some point
     my @path = @_path;
     my $file = canonpath(pop(@path));
     return $file unless ?@path;
@@ -94,9 +94,9 @@ sub catpath (Str $volume, Str $directory, Str $file) returns Str is export {
 ## Misc
 
 sub canonpath (Str $_path) returns Str is export {
-    # take a copy of our args here, maybe 
-    # replace this with 'is copy' parameter 
-    # trait at some point    
+    # take a copy of our args here, maybe
+    # replace this with 'is copy' parameter
+    # trait at some point
     my $path = $_path;
     my $orig_path = $path;
     $path ~~ s:perl5{^([a-z]:)}{{uc$0}};
@@ -158,24 +158,24 @@ sub file_name_is_absolute (Str $file) returns Bool is export {
 
 # This HACK is worse than
 # the File::Spec platform hack
-#sub cwd returns Str {
+#sub cwd returns Str is export {
 #    my @retval = system("cd");
 #    my $cwd = @retval[0];
 #    chomp($cwd);
 #    return $cwd;
 #}
-sub cwd returns Str {
+sub cwd returns Str is export {
   return File::Spec::cwd();
 }
 
-sub tmpdir returns Str {
+sub tmpdir returns Str is export {
   return File::Spec::tmpdir();
 }
 
-sub rel2abs (Str $_path, Str ?$_base) returns Str {
-    # take a copy of our args here, maybe 
-    # replace this with 'is copy' parameter 
-    # trait at some point    
+sub rel2abs (Str $_path, Str ?$_base) returns Str is export {
+    # take a copy of our args here, maybe
+    # replace this with 'is copy' parameter
+    # trait at some point
     my $path = $_path;
     if (!file_name_is_absolute($path)) {
         my $base;
@@ -198,9 +198,9 @@ sub rel2abs (Str $_path, Str ?$_base) returns Str {
 sub abs2rel (Str $_path, Str ?$_base) returns Str is export {
     my $base;
     if (defined($_base) && $_base ne '') {
-        # take a copy of our args here, maybe 
-        # replace this with 'is copy' parameter 
-        # trait at some point        
+        # take a copy of our args here, maybe
+        # replace this with 'is copy' parameter
+        # trait at some point
         $base = $_base;
     }
     else {
@@ -309,4 +309,4 @@ it under the same terms as Perl itself.
 
 See http://www.perl.com/perl/misc/Artistic.html
 
-=cut
+=cut
\ No newline at end of file
