@@ -46,6 +46,7 @@ import Pugs.Pretty
 import Pugs.Types
 import Pugs.Prim.List (op2Fold)
 import Pugs.External
+import Pugs.Embed.Perl5
 
 {-|
 Construct a new, initially empty 'Env' (evaluation environment).
@@ -107,14 +108,15 @@ evaluateMain exp = do
     initAV   <- evalVar "@?INIT"
     initSubs <- fromVals initAV
     enterContext CxtVoid $ do
-        mapM_ evalExp [ App (Val sub) [] [] | sub <- initSubs ]
+        mapM_ evalExp [ App (Val sub) Nothing [] | sub <- initSubs ]
     -- The main runtime
     val      <- resetT $ evaluate exp
     -- S04: END {...}       at run time, ALAP
     endAV    <- evalVar "@*END"
     endSubs  <- fromVals endAV
     enterContext CxtVoid $ do
-        mapM_ evalExp [ App (Val sub) [] [] | sub <- endSubs ]
+        mapM_ evalExp [ App (Val sub) Nothing [] | sub <- endSubs ]
+    liftIO $ performGC
     return val
 
 -- | Evaluate an expression. This function mostly just delegates to 'reduce'.
@@ -132,7 +134,8 @@ findSyms :: Var -> Eval [(String, Val)]
 findSyms name = do
     lex  <- asks envLexical
     glob <- askGlobal
-    let names = [name, toGlobal name]
+    pkg  <- asks envPackage
+    let names = nub [name, toPackage pkg name, toGlobal name]
     syms <- forM [lex, glob] $ \pad -> do
         forM names $ \name' -> do
             case lookupPad name' pad of
@@ -175,7 +178,8 @@ evalVar name = do
     v <- findVar name
     case v of
         Just var -> readRef var
-        Nothing  -> retError "Undeclared variable" name
+        _ | (':':rest) <- name -> return $ VType (mkType rest)
+        _ -> retError "Undeclared variable" name
 
 enterLValue :: Eval a -> Eval a
 enterLValue = local (\e -> e{ envLValue = True })
@@ -187,7 +191,7 @@ findVar name = do
     rv <- findVarRef name
     case rv of
         Nothing  -> case name of
-            ('&':_) -> maybeM (findSub name [] []) $ \sub -> do
+            ('&':_) -> maybeM (findSub name Nothing []) $ \sub -> do
                 return $ codeRef sub
             _ -> return Nothing
         Just ref -> fmap Just $ liftSTM (readTVar ref)
@@ -296,8 +300,10 @@ reduce (Val v) = retVal v
 -- Reduction for variables
 reduce (Var name) = do
     v <- findVar name
-    if isNothing v then retError "Undeclared variable" name else do
-    evalRef (fromJust v)
+    case v of
+        Just var -> evalRef var
+        _ | (':':rest) <- name -> return $ VType (mkType rest)
+        _ -> retError "Undeclared variable" name
 
 reduce (Stmts this rest) | Noop <- unwrap rest = reduce this
 reduce (Stmts this rest) | Noop <- unwrap this = reduce rest
@@ -399,7 +405,7 @@ reduce exp@(Syn name exps) = case name of
                 let (these, rest) = arity `splitAt` vs
                 genSymCC "&next" $ \symNext -> do
                     genSymPrim "&redo" (const $ runBody vs sub') $ \symRedo -> do
-                        apply (updateSubPad sub' (symRedo . symNext)) [] $
+                        apply (updateSubPad sub' (symRedo . symNext)) Nothing $
                             map (Val . VRef . MkRef) these
                 runBody rest sub'
         genSymCC "&last" $ \symLast -> do
@@ -412,7 +418,7 @@ reduce exp@(Syn name exps) = case name of
         sub     <- fromVal =<< evalExp exp
         av      <- newArray []
         symTake <- genSym "@?TAKE" (MkRef av)
-        apply (updateSubPad sub symTake) [] []
+        apply (updateSubPad sub symTake) Nothing []
         fmap VList $ readIVar av
     "loop" -> do
         let [pre, cond, post, body] = case exps of { [_] -> exps'; _ -> exps }
@@ -439,18 +445,22 @@ reduce exp@(Syn name exps) = case name of
         let [match, body] = exps
         break  <- evalVar "&?BLOCK_EXIT"
         vbreak <- fromVal break
-        result <- reduce (App (Var "&infix:~~") [(Var "$_"), match] [])
+        result <- reduce $ case unwrap match of
+            App _ (Just (Var "$_")) _ -> match
+            _ -> App (Var "&infix:~~") Nothing [(Var "$_"), match]
         rb     <- fromVal result
         if rb
-            then enterWhen (subBody vbreak) $ apply vbreak [body] []
+            then enterWhen (subBody vbreak) $ apply vbreak Nothing [body]
             else retVal undef
     "default" -> do
         let [body] = exps
         break  <- evalVar "&?BLOCK_EXIT"
         vbreak <- fromVal break
-        enterWhen (subBody vbreak) $ apply vbreak [body] []
-    "while" -> doWhileUntil id
-    "until" -> doWhileUntil not
+        enterWhen (subBody vbreak) $ apply vbreak Nothing [body]
+    "while" -> doWhileUntil id False
+    "until" -> doWhileUntil not False
+    "postwhile" -> doWhileUntil id True
+    "postuntil" -> doWhileUntil not True
     "=" -> do
         let [lhs, rhs] = exps
         refVal  <- enterLValue $ evalExp lhs
@@ -529,11 +539,11 @@ reduce exp@(Syn name exps) = case name of
         retItem $ VRef av
     -- XXX evil hack for infinite slices
     "[]" | [lhs, App (Var "&postfix:...") invs args] <- unwrap exps
-         , [idx] <- invs ++ args
+         , [idx] <- maybeToList invs ++ args
 --       , not (envLValue env)
          -> reduce (Syn "[...]" [lhs, idx])
     "[]" | [lhs, App (Var "&infix:..") invs args] <- unwrap exps
-         , [idx, Val (VNum n)] <- invs ++ args
+         , [idx, Val (VNum n)] <- maybeToList invs ++ args
          , n == 1/0
 --       , not (envLValue env)
          -> reduce (Syn "[...]" [lhs, idx])
@@ -656,7 +666,7 @@ reduce exp@(Syn name exps) = case name of
     syn | last syn == '=' -> do
         let [lhs, exp] = exps
             op = "&infix:" ++ init syn
-        evalExp $ Syn "=" [lhs, App (Var op) [lhs, exp] []]
+        evalExp $ Syn "=" [lhs, App (Var op) Nothing [lhs, exp]]
     _ -> retError "Unknown syntactic construct" exp
     where
     doCond :: (Bool -> Bool) -> Eval Val
@@ -668,10 +678,14 @@ reduce exp@(Syn name exps) = case name of
             then reduce bodyIf
             else reduce bodyElse
     -- XXX This treatment of while/until loops probably needs work
-    doWhileUntil :: (Bool -> Bool) -> Eval Val
-    doWhileUntil f = do
+    doWhileUntil :: (Bool -> Bool) -> Bool -> Eval Val
+    doWhileUntil f postloop = do
         let [cond, body] = exps
-        enterLoop . fix $ \runLoop -> do
+        -- XXX redo for initial run
+        if postloop
+            then reduce body
+            else retEmpty
+        enterWhile . fix $ \runLoop -> do
             vbool <- enterEvalContext (cxtItem "Bool") cond
             vb    <- fromVal vbool
             case f vb of
@@ -685,37 +699,37 @@ reduce exp@(Syn name exps) = case name of
 
 -- XXX absolutely evil bloody hack for context hinters
 reduce (App (Var "&hash") invs args) =
-    enterEvalContext cxtItemAny $ Syn "\\{}" [Syn "," $ invs ++ args]
+    enterEvalContext cxtItemAny $ Syn "\\{}" [Syn "," $ maybeToList invs ++ args]
 
 reduce (App (Var "&list") invs args) =
-    enterEvalContext cxtSlurpyAny $ case invs ++ args of
+    enterEvalContext cxtSlurpyAny $ case maybeToList invs ++ args of
         []    -> Val (VList [])
         [exp] -> exp
         exps  -> Syn "," exps
 
 reduce (App (Var "&scalar") invs args)
-    | [exp] <- invs ++ args = enterEvalContext cxtItemAny exp
-    | otherwise = enterEvalContext cxtItemAny $ Syn "," (invs ++ args)
+    | [exp] <- maybeToList invs ++ args = enterEvalContext cxtItemAny exp
+    | otherwise = enterEvalContext cxtItemAny $ Syn "," (maybeToList invs ++ args)
 
 -- XXX absolutely evil bloody hack for "zip"
 reduce (App (Var "&zip") invs args) = do
-    vals <- mapM (enterRValue . enterEvalContext (cxtItem "Array")) (invs ++ args)
+    vals <- mapM (enterRValue . enterEvalContext (cxtItem "Array")) (maybeToList invs ++ args)
     val  <- op0Zip vals
     retVal val
 
--- XXX absolutely evil bloody hack for "goto"
-reduce (App (Var "&not") [] []) = retEmpty
+-- XXX absolutely evil bloody hack for "not"
+reduce (App (Var "&not") Nothing []) = retEmpty
 
 reduce (App (Var "&not") invs args) = do
-    bool <- fromVal =<< evalExp (last $ invs ++ args)
+    bool <- fromVal =<< evalExp (last $ maybeToList invs ++ args)
     retVal $ VBool (not bool)
 
 -- XXX absolutely evil bloody hack for "goto"
-reduce (App (Var "&goto") (subExp:invs) args) = do
+reduce (App (Var "&goto") (Just subExp) args) = do
     vsub <- enterEvalContext (cxtItem "Code") subExp
     sub <- fromVal vsub
     local callerEnv $ do
-        val <- apply sub invs args
+        val <- apply sub Nothing args
         shiftT $ const (retVal val)
     where
     callerEnv :: Env -> Env
@@ -727,63 +741,60 @@ reduce (App (Var "&goto") (subExp:invs) args) = do
            }
 
 -- XXX absolutely evil bloody hack for "assuming"
-reduce (App (Var "&assuming") (subExp:invs) args) = do
+reduce (App (Var "&assuming") (Just subExp) args) = do
     vsub <- enterEvalContext (cxtItem "Code") subExp
     sub <- fromVal vsub
-    case bindSomeParams sub invs args of
+    case bindSomeParams sub Nothing args of
         Left errMsg      -> fail errMsg
         Right curriedSub -> retVal $ castV $ curriedSub
 
-reduce (App (Var "&infix:=>") invs args) = reduce (Syn "=>" (invs ++ args))
+reduce (App (Var "&infix:=>") invs args) = reduce (Syn "=>" (maybeToList invs ++ args))
 
 reduce (App (Var name@('&':_)) invs args) = do
     sub     <- findSub name invs args
     case sub of
         Just sub    -> applySub sub invs args
-        _ | [Syn "," invs'] <- unwrap invs, null args -> do
-            sub <- findSub name invs' []
+        _ | [Syn "," args'] <- unwrap args -> do
+            sub <- findSub name invs args'
             if isNothing sub then err else do
             fail $ "Extra space found after " ++ name ++ " (...) -- did you mean " ++ name ++ "(...) instead?"
         _ -> err
     where
     err = retError "No compatible subroutine found" name
-    applySub :: VCode -> [Exp] -> [Exp] -> Eval Val
+    applySub :: VCode -> (Maybe Exp) -> [Exp] -> Eval Val
     applySub sub invs args
         -- list-associativity
         | MkCode{ subAssoc = "list" }      <- sub
-        , (App (Var name') invs' []):rest  <- invs
+        , (App (Var name') Nothing args'):rest  <- args
         , name == name'
-        = applySub sub (invs' ++ rest)  []
+        = applySub sub invs (args' ++ rest)
         -- fix subParams to agree with number of actual arguments
         | MkCode{ subAssoc = "list", subParams = (p:_) }   <- sub
-        , null args
-        = apply sub{ subParams = (length invs) `replicate` p } invs []
+        = apply sub{ subParams = (length args) `replicate` p } invs args
         -- chain-associativity
         | MkCode{ subAssoc = "chain" }  <- sub
-        , (App _ _ []):_                <- invs
-        , null args
-        = mungeChainSub sub invs
+        , (App _ _ []):_                <- args
+        = mungeChainSub sub args
         | MkCode{ subAssoc = "chain", subParams = (p:_) }   <- sub
-        = apply sub{ subParams = (length invs) `replicate` p } invs []
+        = apply sub{ subParams = (length args) `replicate` p } invs args
         -- normal application
         | otherwise
         = apply sub invs args
     mungeChainSub :: VCode -> [Exp] -> Eval Val
-    mungeChainSub sub invs = do
+    mungeChainSub sub args = do
         let MkCode{ subAssoc = "chain", subParams = (p:_) } = sub
-            (App (Var name') invs' args'):rest = invs
+            (App (Var name') invs' args'):rest = args
         theSub   <- findSub name' invs' args'
         case theSub of
-            Just sub'    -> applyChainSub sub invs sub' invs' args' rest
-            Nothing      -> apply sub{ subParams = (length invs) `replicate` p } invs [] -- XXX Wrong
-    applyChainSub :: VCode -> [Exp] -> VCode -> [Exp] -> [a] -> [Exp] -> Eval Val
-    applyChainSub sub invs sub' invs' args' rest
+            Just sub'    -> applyChainSub sub args sub' args' rest
+            Nothing      -> apply sub{ subParams = (length args) `replicate` p } Nothing args -- XXX Wrong
+    applyChainSub :: VCode -> [Exp] -> VCode -> [Exp] -> [Exp] -> Eval Val
+    applyChainSub sub args sub' args' rest
         | MkCode{ subAssoc = "chain", subBody = fun, subParams = prm }   <- sub
         , MkCode{ subAssoc = "chain", subBody = fun', subParams = prm' } <- sub'
-        , null args'
-        = applySub sub{ subParams = prm ++ tail prm', subBody = Prim $ chainFun prm' fun' prm fun } (invs' ++ rest) []
+        = applySub sub{ subParams = prm ++ tail prm', subBody = Prim $ chainFun prm' fun' prm fun } Nothing (args' ++ rest)
         | MkCode{ subAssoc = "chain", subParams = (p:_) }   <- sub
-        = apply sub{ subParams = (length invs) `replicate` p } invs [] -- XXX Wrong
+        = apply sub{ subParams = (length args) `replicate` p } Nothing args -- XXX Wrong
         | otherwise
         = internalError "applyChainsub did not match a chain subroutine"
 
@@ -829,25 +840,85 @@ cxtOfExp (App (Var name) invs args)   = do
         _ -> cxtSlurpyAny
 cxtOfExp _                      = return cxtSlurpyAny
 
-findSub :: String -> [Exp] -> [Exp] -> Eval (Maybe VCode)
+findSub :: String -> Maybe Exp -> [Exp] -> Eval (Maybe VCode)
 findSub name' invs args = do
     let name = possiblyFixOperatorName name'
     case invs of
-        [exp] | not (':' `elem` drop 2 name) -> do
-            typ     <- evalExpType exp
-            subs    <- findWithPkg (showType typ) name
-            if isJust subs then return subs else findSub' name
-        _ -> do
-            sub <- findSub' name
-            if isNothing sub then possiblyBuildMetaopVCode name else return sub
+        Just _ | Just (package, name') <- breakOnGlue "::" name
+                 , Just (sig, "") <- breakOnGlue "SUPER" package -> do
+            typ <- asks envPackage
+            findSuperSub (mkType typ) (sig ++ name')
+        Just exp | not (':' `elem` drop 2 name) -> do
+            typ     <- evalInvType $ unwrap exp
+            if typ == mkType "Scalar::Perl5" then runPerl5Sub name else do
+            findTypedSub typ name
+        _ | [exp] <- args -> do
+            typ     <- evalInvType $ unwrap exp
+            findTypedSub typ name
+        _ -> findBuiltinSub name
     where
+    findSuperSub :: Type -> String -> Eval (Maybe VCode)
+    findSuperSub typ name = do
+        let pkg = showType typ
+            qualified = (head name:pkg) ++ "::" ++ tail name
+        subs    <- findWithSuper pkg name
+        subs'   <- if isJust subs then return subs else findBuiltinSub name
+        case subs' of
+            Just sub | subName sub == qualified -> return Nothing
+            _   -> return subs'
+    findTypedSub :: Type -> String -> Eval (Maybe VCode)
+    findTypedSub typ name = do
+        subs    <- findWithPkg (showType typ) name
+        if isJust subs then return subs else findBuiltinSub name
+    findBuiltinSub :: String -> Eval (Maybe VCode)
+    findBuiltinSub name = do
+        sub <- findSub' name
+        if isNothing sub then possiblyBuildMetaopVCode name else return sub
+    evalInvType :: Exp -> Eval Type
+    evalInvType x@(Var (':':typ)) = do
+        typ' <- evalExpType x
+        return $ if typ' == mkType "Scalar::Perl5" then typ' else mkType typ
+    evalInvType (App (Var "&new") (Just inv) _) = do
+        evalInvType $ unwrap inv
+    evalInvType x@(App (Var _) (Just inv) _) = do
+        typ <- evalInvType $ unwrap inv
+        if typ == mkType "Scalar::Perl5" then return typ else evalExpType x
+    evalInvType x = evalExpType $ unwrap x
+    runPerl5Sub :: String -> Eval (Maybe VCode)
+    runPerl5Sub name = do
+        metaSub <- possiblyBuildMetaopVCode name
+        if isJust metaSub then return metaSub else do
+        return . Just $ mkPrim
+            { subName     = name
+            , subParams   = makeParams ["Object", "List", "Named"]
+            , subReturns  = mkType "Scalar::Perl5"
+            , subBody     = Prim $ \(inv:named:pos:_) -> do
+                sv      <- fromVal inv
+                posSVs  <- fromVals pos
+                namSVs  <- fmap concat (fromVals named)
+                let svs = posSVs ++ namSVs
+                found   <- liftIO $ canPerl5 sv (tail name)
+                found'  <- liftIO $ if found
+                    then return found
+                    else canPerl5 sv "AUTOLOAD"
+                if not found' then evalExp (App (Var name) Nothing (map (Val . PerlSV) (sv:svs))) else do
+                env     <- ask
+                rv      <- liftIO $ do
+                    envSV   <- mkVal (VControl $ ControlEnv env)
+                    subSV   <- vstrToSV $ tail name
+                    invokePerl5 subSV sv svs envSV (enumCxt $ envContext env)
+                return $ case rv of
+                    [sv]    -> PerlSV sv
+                    _       -> VList (map PerlSV rv)
+            }
+    possiblyBuildMetaopVCode :: String -> Eval (Maybe VCode)
     possiblyBuildMetaopVCode op' | "&prefix:[" `isPrefixOf` op', "]" `isSuffixOf` op' = do 
         -- Strip the trailing "]" from op
         let op = drop 9 (init op')
         -- We try to find the userdefined sub.
         -- We use the first two elements of invs as invocants, as these are the
         -- types of the op.
-            rv = findSub ("&infix:" ++ op) (take 2 (invs ++ [Val undef, Val undef])) []
+            rv = findSub ("&infix:" ++ op) Nothing (take 2 $ args ++ [Val undef, Val undef])
         maybeM rv $ \code -> return $ mkPrim
             { subName     = "&prefix:[" ++ op ++ "]"
             , subType     = SubPrim
@@ -864,7 +935,7 @@ findSub name' invs args = do
         possiblyBuildMetaopVCode ("&prefix:" ++ op ++ "<<")
     possiblyBuildMetaopVCode op' | "&prefix:" `isPrefixOf` op', "<<" `isSuffixOf` op' = do 
         let op = drop 8 (init (init op'))
-            rv = findSub ("&prefix:" ++ op) [head $ invs ++ [Val undef]] []
+            rv = findSub ("&prefix:" ++ op) Nothing [head $ args ++ [Val undef]]
         maybeM rv $ \code -> return $ mkPrim
             { subName     = "&prefix:" ++ op ++ "<<"
             , subType     = SubPrim
@@ -879,7 +950,7 @@ findSub name' invs args = do
         possiblyBuildMetaopVCode ("&postfix:>>" ++ op)
     possiblyBuildMetaopVCode op' | "&postfix:>>" `isPrefixOf` op' = do
         let op = drop 11 op'
-            rv = findSub ("&postfix:" ++ op) [head $ invs ++ [Val undef]] []
+            rv = findSub ("&postfix:" ++ op) Nothing [head $ args ++ [Val undef]]
         maybeM rv $ \code -> return $ mkPrim
             { subName     = "&postfix:>>" ++ op
             , subType     = SubPrim
@@ -894,7 +965,7 @@ findSub name' invs args = do
         possiblyBuildMetaopVCode ("&infix:>>" ++ op ++ "<<")
     possiblyBuildMetaopVCode op' | "&infix:>>" `isPrefixOf` op', "<<" `isSuffixOf` op' = do 
         let op = drop 9 (init (init op'))
-            rv = findSub ("&infix:" ++ op) (take 2 (invs ++ [Val undef, Val undef])) []
+            rv = findSub ("&infix:" ++ op) Nothing (take 2 (args ++ [Val undef, Val undef]))
         maybeM rv $ \code -> return $ mkPrim
             { subName     = "&infix:>>" ++ op ++ "<<"
             , subType     = SubPrim
@@ -915,20 +986,26 @@ findSub name' invs args = do
             meta    <- readRef ref
             fetch   <- doHash meta hash_fetchVal
             fromVal =<< fetch "traits"
+    findWithPkg :: String -> String -> Eval (Maybe VCode)
     findWithPkg pkg name = do
         subs <- findSub' (('&':pkg) ++ "::" ++ tail name)
         if isJust subs then return subs else do
+        findWithSuper pkg name
+    findWithSuper :: String -> String -> Eval (Maybe VCode)
+    findWithSuper pkg name = do
         -- get superclasses
-        attrs <- findAttrs pkg
-        if isNothing attrs then findSub' name else do
+        attrs <- fmap (fmap (filter (/= pkg) . nub)) $ findAttrs pkg
+        if isNothing attrs || null (fromJust attrs) then findSub' name else do
         (`fix` (fromJust attrs)) $ \run pkgs -> do
             if null pkgs then return Nothing else do
             subs <- findWithPkg (head pkgs) name
             if isJust subs then return subs else run (tail pkgs)
+    findSub' :: String -> Eval (Maybe VCode)
     findSub' name = do
         subSyms     <- findSyms name
-        lens        <- mapM argSlurpLen (unwrap $ invs ++ args)
+        lens        <- mapM argSlurpLen (unwrap $ maybeToList invs ++ args)
         doFindSub (sum lens) subSyms
+    argSlurpLen :: Exp -> Eval Int
     argSlurpLen (Val listMVal) = do
         listVal  <- fromVal listMVal
         return $ length (vCast listVal :: [Val])
@@ -938,6 +1015,7 @@ findSub name' invs args = do
         return $ length (vCast listVal :: [Val])
     argSlurpLen (Syn "," list) =  return $ length list
     argSlurpLen _ = return 1 -- XXX
+    doFindSub :: Int -> [(String, Val)] -> Eval (Maybe VCode)
     doFindSub slurpLen subSyms = do
         subs' <- subs slurpLen subSyms
         -- let foo (x, sub) = show x ++ show (map paramContext $ subParams sub)
@@ -945,18 +1023,19 @@ findSub name' invs args = do
         return $ case sort subs' of
             ((_, sub):_)    -> Just sub
             _               -> Nothing
-    subs slurpLen subSyms = (liftM catMaybes) $ (`mapM` subSyms) $ \(n, val) -> do
-        sub@(MkCode{ subType = subT, subReturns = ret, subParams = prms }) <- fromVal val
-        let isGlobal = '*' `elem` n
-        let rv = return $ arityMatch sub (length (invs ++ args)) slurpLen
+    subs :: Int -> [(String, Val)] -> Eval [((Bool, Bool, Int, Int), VCode)]
+    subs slurpLen subSyms = (liftM catMaybes) $ (`mapM` subSyms) $ \(_, val) -> do
+        sub@(MkCode{ subReturns = ret, subParams = prms }) <- fromVal val
+        let rv = return $ arityMatch sub (length (maybeToList invs ++ args)) slurpLen
         maybeM rv $ \fun -> do
             -- if deltaFromCxt ret == 0 then return Nothing else do
             let pairs = map (typeOfCxt . paramContext) prms
-                            `zip` (map unwrap $ invs ++ args)
+                            `zip` (map unwrap $ maybeToList invs ++ args)
             deltaCxt    <- deltaFromCxt ret
             deltaArgs   <- mapM deltaFromPair pairs
             let bound = either (const False) (const True) $ bindParams sub invs args
-            return ((isGlobal, subT, isMulti sub, bound, sum deltaArgs, deltaCxt), fun)
+            return ((isMulti sub, bound, sum deltaArgs, deltaCxt), fun)
+    deltaFromCxt :: Type -> Eval Int
     deltaFromCxt x  = do
         cls <- asks envClasses
         cxt <- asks envContext
@@ -976,7 +1055,7 @@ evalExpType (Val val) = evalValType val
 evalExpType (App (Val val) _ _) = do
     sub <- fromVal val
     return $ subReturns sub
-evalExpType (App (Var "&new") [(Val (VType typ))] _) = return typ
+evalExpType (App (Var "&new") (Just (Var (':':name))) _) = return $ mkType name
 evalExpType (App (Var name) invs args) = do
     sub <- findSub name invs args
     case sub of
@@ -991,6 +1070,7 @@ evalExpType (Pos _ exp) = evalExpType exp
 evalExpType (Pad _ _ exp) = evalExpType exp
 evalExpType (Sym _ _ exp) = evalExpType exp
 evalExpType (Stmts _ exp) = evalExpType exp
+evalExpType (Syn "sub" [exp]) = evalExpType exp
 evalExpType _ = return $ mkType "Any"
 
 chainFun :: Params -> Exp -> Params -> Exp -> [Val] -> Eval Val
@@ -1026,9 +1106,9 @@ Apply a sub (or other code) to lists of invocants and arguments.
 
 Mostly delegates to 'doApply' after explicitly retrieving the local 'Env'.
 -}
-apply :: VCode -- ^ The sub to apply
-      -> [Exp] -- ^ List of invocants
-      -> [Exp] -- ^ List of arguments (non-invocant)
+apply :: VCode       -- ^ The sub to apply
+      -> (Maybe Exp) -- ^ invocant
+      -> [Exp]       -- ^ List of arguments (non-invocant)
       -> Eval Val
 apply sub invs args = do
     env <- ask
@@ -1042,7 +1122,7 @@ specified context.
 -}
 doApply :: Env   -- ^ Environment to evaluate in
         -> VCode -- ^ Code to apply
-        -> [Exp] -- ^ Invocants (arguments before the colon)
+        -> (Maybe Exp) -- ^ Invocants (arguments before the colon)
         -> [Exp] -- ^ Arguments (not including invocants)
         -> Eval Val
 doApply env sub@MkCode{ subCont = cont, subBody = fun, subType = typ } invs args =
@@ -1129,6 +1209,13 @@ doApply env sub@MkCode{ subCont = cont, subBody = fun, subType = typ } invs args
         | isaType (envClasses env) "Junction" typ    = True
         | otherwise                     = False
 
+toPackage :: String -> String -> String
+toPackage pkg name
+    | (sigil, identifier) <- break (\x -> isAlpha x || x == '_') name
+    , last sigil /= '*'
+    = concat [sigil, pkg, "::", identifier]
+    | otherwise = name
+
 toGlobal :: String -> String
 toGlobal name
     | (sigil, identifier) <- break (\x -> isAlpha x || x == '_') name
@@ -1156,10 +1243,11 @@ arityMatch sub@MkCode{ subAssoc = assoc, subParams = prms } argLen argSlurpLen
     = Nothing
 
 doFetch :: (Val -> Eval (IVar VScalar))
-            -> (Val -> Eval Val)
-            -> (forall v. (Value v) => Eval v)
-            -> Bool -> Bool
-            -> Eval Val
+        -> (Val -> Eval Val)
+        -> (forall v. (Value v) => Eval v)
+        -> Bool
+        -> Bool
+        -> Eval Val
 doFetch fetchElem fetchVal fetchIdx isLV isSV = case (isLV, isSV) of
     (True, True) -> do
         -- LValue, Scalar context
