@@ -34,6 +34,8 @@ module Pugs.Internals (
     module System.Directory,
     module System.Cmd,
     module System.Process,
+    module System.Mem,
+    module System.Mem.Weak,
     module Control.Monad.RWS,
     module Control.Monad.Error,
     module Control.Concurrent,
@@ -67,6 +69,7 @@ module Pugs.Internals (
     unsafePerformSTM,
     possiblyFixOperatorName,
     maybeM,
+    safeMode,
 ) where
 
 import UTF8
@@ -84,13 +87,15 @@ import System.Time
 import System.Cmd
 import System.Process
 import System.IO (
-    Handle, stdin, stdout, hClose, hGetLine, hGetContents,
-    openFile, hPutStr, hPutStrLn, IOMode(..), stderr, SeekMode(..),
+    Handle, stdin, stdout, hClose, hGetLine, hGetChar, hGetContents,
+    openFile, hSetBinaryMode, hPutStr, hPutStrLn, IOMode(..), stderr, SeekMode(..),
     hSetBuffering, BufferMode(..), hIsTerminalDevice, hFlush, hPrint, isEOF
     )
 import System.IO.Unsafe
 import System.IO.Error (ioeGetErrorString, isUserError)
 import System.Directory
+import System.Mem
+import System.Mem.Weak
 import Control.Exception (catchJust, errorCalls)
 import Control.Monad.RWS
 import Control.Monad.Error (MonadError(..))
@@ -127,6 +132,10 @@ instance Eq (a -> b) where
     _ == _ = False
 instance Ord (a -> b) where
     compare _ _ = LT
+instance Eq Dynamic where
+    _ == _ = False
+instance Ord Dynamic where
+    compare _ _ = LT
 
 internalError :: String -> a
 internalError s = error $
@@ -164,22 +173,56 @@ decodeUTF8 str = fst $ decode bytes
     where
     bytes = map (toEnum . ord) str
 
-forM :: (Monad m) => [a] -> (a -> m b) -> m [b]
+{-|
+Take a list of values, and a monad-producing function, and apply that function
+to each element of the list. The resulting monads are combined into a single
+monad producing a list of the resulting values.
+
+(This is just @mapM@ with the arguments reversed.)
+-}
+forM :: (Monad m) 
+     => [a]        -- ^ List of values to loop over
+     -> (a -> m b) -- ^ The \'body\' of the for loop
+     -> m [b]      -- ^ Monad containing a list of the results
 forM = flip mapM
 
-forM_ :: (Monad m) => [a] -> (a -> m b) -> m ()
+{-|
+Take a list of values, and a monad-producing function, and apply that function
+to each element of the list in sequence. The values produced by the monadic
+function are discarded.
+
+(This is just @mapM_@ with the arguments reversed.)
+-}
+forM_ :: (Monad m) 
+      => [a]        -- ^ List of values to loop over
+      -> (a -> m b) -- ^ The \'body\' of the for loop
+      -> m ()
 forM_ = flip mapM_
 
 tryIO :: (MonadIO m) => a -> IO a -> m a
 tryIO err = liftIO . (`catch` (const $ return err))
 
-combine :: [a->a] -> a -> a
+{-|
+Compose a list of @(a -> a)@ transformer functions into a single chained
+function, using @foldr@ via the @(.)@ operator.
+
+Note that the transformations are applied to the eventual argument in 
+right-to-left order.
+-}
+combine :: [a -> a] -- ^ List of transformer functions
+        -> (a -> a) -- ^ The final combined transformer
 combine = foldr (.) id
 
 unsafePerformSTM :: STM a -> a
 unsafePerformSTM = unsafePerformIO . atomically
 
-modifyTVar :: TVar a -> (a -> a) -> STM ()
+{-|
+Read an STM variable, apply some transformation function to it, and write the
+transformed value back to the same variable.
+-}
+modifyTVar :: TVar a 
+           -> (a -> a) 
+           -> STM ()
 modifyTVar var f = do
     x <- readTVar var
     writeTVar var (f x)
@@ -187,12 +230,28 @@ modifyTVar var f = do
 -- instance MonadIO STM where
 --     liftIO = unsafeIOToSTM
 
-maybeM :: (FunctorM f, Monad m) => m (f a) -> (a -> m b) -> m (f b)
+{-|
+Extract a @Maybe@ value from the first argument (a monad).
+
+If it's a @Just@ (i.e. it contains a value), apply the second argument
+(a monad-producing function) to it, and @return@ the contents of /that/ 
+wrapped in a @Just@.
+
+Otherwise, merely @return Nothing@.
+
+(Strictly speaking, this function can operate with any @FunctorM@, not just
+@Maybe@, but it helps to have a concrete example to explain things.)
+-}
+maybeM :: (FunctorM f, Monad m) 
+       => m (f a)    -- ^ A @Maybe@ value encapsulated in a monad
+       -> (a -> m b) -- ^ Action to perform on the first arg /if/ it contains
+                     --     a value
+       -> m (f b)    -- ^ Monad containing (@Just@ /result/) or @Nothing@
 maybeM f m = fmapM m =<< f
 
 {-|
-Transform an operator name, for example @&infix:<+>@ or @&prefix:«[+]»@, into
-its internal name (@&infix:+@ and @&prefix:[+]@ respectively).
+Transform an operator name, for example @&infix:\<+\>@ or @&prefix:«[+]»@, 
+into its internal name (@&infix:+@ and @&prefix:[+]@ respectively).
 -}
 possiblyFixOperatorName :: String -> String
 possiblyFixOperatorName name
@@ -219,3 +278,15 @@ possiblyFixOperatorName name
     -- <bar> --> bar
     dropBrackets ('<':(rest@(_:_)))       = if (last rest) == '>' then init rest else '<':rest
     dropBrackets x                        = x
+
+{-|
+Returns @True@ if the environment variable @PUGS_SAFEMODE@ is set to a
+true value. Most IO primitives are disabled under safe mode.
+-}
+{-# NOINLINE safeMode #-}
+safeMode :: Bool
+safeMode = case (unsafePerformIO $ getEnv "PUGS_SAFEMODE") of
+    Nothing     -> False
+    Just ""     -> False
+    Just "0"    -> False
+    _           -> True
