@@ -10,7 +10,22 @@
 >   In the Land of Mordor where the Shadows lie.
 -}
 
-module Pugs.Types where
+module Pugs.Types (
+    Type(..), mkType, anyType, showType,
+    ClassTree,
+
+    Cxt(..), 
+    cxtItem, cxtSlurpy, cxtVoid, cxtItemAny, cxtSlurpyAny,
+    typeOfCxt, isSlurpyCxt, isItemCxt, isVoidCxt,
+    enumCxt, cxtEnum,
+
+    Var, 
+
+    VStr, VBool, VInt, VRat, VNum, VComplex, VHandle, VSocket,
+    VThread(..), VRule(..),
+
+    MatchPGE(..)
+) where
 import Pugs.Internals
 
 data Type
@@ -51,31 +66,53 @@ typeOfCxt CxtVoid           = anyType
 typeOfCxt (CxtItem typ)     = typ
 typeOfCxt (CxtSlurpy typ)   = typ
 
--- |Return a 'Cxt' indicating a context expecting a scalar of any type
+-- | Return a 'Cxt' indicating a context expecting a scalar of any type
 cxtItemAny :: Cxt
 cxtItemAny   = CxtItem anyType
--- |Return a 'Cxt' indicating a context expecting a list of any type
+-- | Return a 'Cxt' indicating a context expecting a list of any type
 cxtSlurpyAny :: Cxt
 cxtSlurpyAny = CxtSlurpy anyType
 
--- |Return true if the given 'Cxt' (context) is 'CxtSlurpy', rather than
--- 'CxtItem' or 'CxtVoid'.
+{-|
+Return true if the given 'Cxt' (context) is 'CxtSlurpy', rather than
+'CxtItem' or 'CxtVoid'.
+-}
 isSlurpyCxt :: Cxt -> Bool
 isSlurpyCxt (CxtSlurpy _) = True
 isSlurpyCxt _             = False
--- |Return true if the given 'Cxt' (context) is 'CxtItem', rather than
--- 'CxtSlurpy' or 'CxtVoid'.
+{-|
+Return true if the given 'Cxt' (context) is 'CxtItem', rather than
+'CxtSlurpy' or 'CxtVoid'.
+-}
 isItemCxt :: Cxt -> Bool
 isItemCxt   (CxtItem _)   = True
 isItemCxt   _             = False
--- |Return true if the given 'Cxt' (context) is 'CxtVoid', rather than
--- 'CxtSlurpy' or 'CxtItem'.
+{-|
+Return true if the given 'Cxt' (context) is 'CxtVoid', rather than
+'CxtSlurpy' or 'CxtItem'.
+-}
 isVoidCxt :: Cxt -> Bool
 isVoidCxt   CxtVoid       = True
 isVoidCxt   _             = False
 
--- |Make a type value representing the type with the specified name.
--- Recognises conjunctive (&) and disjunctive (|) types.
+-- | Return the Perl 5 calling convention bit value for the context.
+enumCxt :: (Num a) => Cxt -> a
+enumCxt CxtVoid       = 128
+enumCxt (CxtItem _)   = 0
+enumCxt (CxtSlurpy _) = 1
+
+-- | Return the 'Cxt' corresponding to the given P5 calling convention bits.
+cxtEnum :: (Show a, Num a) => a -> Cxt
+cxtEnum 128 = CxtVoid
+cxtEnum 0   = cxtItemAny
+cxtEnum 1   = cxtSlurpyAny
+cxtEnum n   = error ("Invalid cxt: " ++ show n)
+
+{-|
+Make a type value representing the type with the specified name.
+
+Recognises conjunctive (&) and disjunctive (|) types.
+-}
 mkType :: String -- ^ Name of the type, e.g. \"Hash\" or \"Str|Int\"
        -> Type
 mkType str
@@ -86,32 +123,32 @@ mkType str
     | otherwise
     = MkType str
 
--- |Variable name
+-- | Variable name.
 type Var   = String
--- |Uses Haskell's underlying representation for strings.
+-- | Uses Haskell's underlying representation for strings.
 type VStr  = String
--- |Uses Haskell's underlying representation for booleans.
+-- | Uses Haskell's underlying representation for booleans.
 type VBool = Bool
--- |Uses Haskell's underlying representation for integers.
+-- | Uses Haskell's underlying representation for integers.
 type VInt  = Integer
--- |Uses Haskell's underlying representation for rational numbers.
+-- | Uses Haskell's underlying representation for rational numbers.
 type VRat  = Rational
--- |Uses Haskell's 'Double' type to represent arbitrary numbers.
+-- | Uses Haskell's 'Double' type to represent arbitrary numbers.
 type VNum  = Double
--- |Uses Haskell's underlying representation for complex numbers.
+-- | Uses Haskell's underlying representation for complex numbers.
 type VComplex = Complex VNum
--- |Uses Haskell's underlying representation for filehandles.
+-- | Uses Haskell's underlying representation for filehandles.
 type VHandle = Handle
--- |Uses Haskell's underlying representation for sockets.
+-- | Uses Haskell's underlying representation for sockets.
 type VSocket = Socket
--- |Uses Haskell's underlying representation for threads.
+-- | Uses Haskell's underlying representation for threads.
 data VThread a = MkThread
     { threadId      :: ThreadId
     , threadLock    :: TMVar a
     }
     deriving (Show, Eq, Ord, Typeable)
 
--- |Rule Match object from PGE
+-- | Rule Match object from PGE
 data MatchPGE
     = PGE_Match !Int !Int ![MatchPGE] ![(VStr, MatchPGE)]
     | PGE_Array ![MatchPGE]
@@ -119,20 +156,25 @@ data MatchPGE
     deriving (Show, Eq, Ord, Read, Typeable)
 
 
--- |Representation for rules (i.e. regexes). Currently consists of a
--- "RRegex" 'Regex', and a boolean flag indicating whether the rule has
--- \'global\' semantics (Perl5's \/g), i.e. whether it matches all occurences
--- or just the first.
+{-|
+Representation for rules (i.e. regexes).
+
+Currently there are two types of rules: Perl 5 rules, implemented with PCRE,
+and Perl 6 rules, implemented with PGE.
+-}
 data VRule
+    -- | Perl5-compatible regular expression
     = MkRulePCRE
-        { rxRegex     :: !Regex -- ^ The \'regular\' expression
+        { rxRegex     :: !Regex -- ^ The \'regular\' expression (as a PCRE
+                                --     'Regex' object)
         , rxGlobal    :: !Bool  -- ^ Flag indicating \'global\' (match-all)
-	, rxStringify :: !Bool
+	    , rxStringify :: !Bool
         }
+    -- | Parrot Grammar Engine rule
     | MkRulePGE
-        { rxRule      :: !String -- ^ The \'rule\' expression
+        { rxRule      :: !String -- ^ The rule string
         , rxGlobal    :: !Bool   -- ^ Flag indicating \'global\' (match-all)
-	, rxStringify :: !Bool
+	    , rxStringify :: !Bool
         }
     deriving (Show, Eq, Ord, Typeable)
 
