@@ -1,7 +1,7 @@
 module Dummy;
 use v6;
 
-sub load_test () is exported {
+sub load_test () is export {
   say "Module Dummy.pm was loaded"
 };
 
